def format_sizeof(num, suffix='', divisor=1000):
    """
    Formats a number (greater than unity) with SI Order of Magnitude
    prefixes.
    Parameters
    ----------
    num  : float
        Number ( >= 1) to format.
    suffix  : str, optional
        Post-postfix [default: ''].
    divisor  : float, optionl
        Divisor between prefixes [default: 1000].
    Returns
    -------
    out  : str
        Number with Order of Magnitude SI unit postfix.
    """
    for unit in ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(num) < 999.95:
            if abs(num) < 99.95:
                if abs(num) < 9.995:
                    return '{0:1.2f}'.format(num) + unit + suffix
                return '{0:2.1f}'.format(num) + unit + suffix
            return '{0:3.0f}'.format(num) + unit + suffix
        num /= divisor
    return '{0:3.1f}Y'.format(num) + suffix



if __name__ == '__main__':
    print(format_sizeof(547294757398, 'B'))



#!/usr/bin/env python3

from io import BytesIO
import os
from pathlib import Path
from typing import NamedTuple

from .xmlparse import process_xml
from .hash import sha1sum_from_file
from .util import format_sizeof
from .curl import PROPFIND, download_file, init_curl

from .settings import SETTINGS, load_settings_file, NoConfigFoundError
from .messages import config_file_help

DESTINATION = None


class File(NamedTuple):
    path: str
    etag: str
    size: int

def verify_file(full_path, etag):
    """Verify the given file is correct

    etag is an sha1sum, optionally enclosed in double quotes"""

    digest = etag.replace('"', '') # kill "'s
    sha1 = sha1sum_from_file(full_path)

    return sha1 == digest

def download_files(files):
    for file in files:
        if not try_a_hundred_times(file):
            raise Exception("Failed to download file")

def try_a_hundred_times(file):
    for t in range(1000):
        if download_and_verify(file):
            return True

    return False


def download_and_verify(file):
    full_filename = file.path
    short_filename = os.path.basename(full_filename)

    print(f"Downloading {short_filename}...")

    output_file = DESTINATION / short_filename
    with output_file.open('wb') as outf:
        download_file(full_filename, outf)

    return verify_file(output_file, file.etag)

def get_folders():
    buffer = BytesIO()
    PROPFIND('dav/', buffer)

    folders = []
    for item in process_xml(buffer.getvalue()):
        if item['filename'] == '/dav/':
            continue
        if 'getcontenttype' in item: # indicates a file
            continue

        folders.append(item['filename'])
    return folders


def choose_folder():
    folders = get_folders()
    for i, folder in enumerate(folders):
        print(f"{i}. {folder}")

    choice = input("\nPlease enter folder choice (or q to abort): ")

    try:
        chosen = folders[int(choice)]
    except:
        exit()

    return chosen

def get_files(folder):
    buffer = BytesIO()
    PROPFIND(folder, buffer)

    files = []
    for item in process_xml(buffer.getvalue()):
        if 'getcontenttype' not in item: # indicates a folder
            continue

        files.append(File(item['filename'],
                          item['getetag'],
                          int(item['getcontentlength'])))
    return set(files)

def print_file_report(folder, files):

    files_size = format_sizeof(sum([f.size for f in files]), 'B')
    files_count = len(files)
    files_head = list(files)[:5]

    print(f"This folder contains {files_count} new files, which will "
          f"consume {files_size}.")
    print("Here are the first few files:")
    for file in files_head:
        file_size = format_sizeof(file.size, 'B')
        print(f"    {file_size}\t{file.path}")

    print("    ....\t....")

def filter_existing_files(files):
    print("Checking for existing files...")
    existing_files = []
    incorrect_files = []

    for file in files:
        test_filename = DESTINATION / os.path.basename(file.path)
        if os.path.exists(test_filename):
            print(f"\tVerifying {test_filename}...")
            if verify_file(test_filename, file.etag):
                existing_files.append(file)
            else:
                incorrect_files.append(file)
                
    if len(incorrect_files) > 0:
        # TODO: print a report of what those files are
        print("\nSome of the files you wish to download already exist "
              "in your destination directory, however their sha1 checksum "
              "does not match what Box has.")

        print("\nThis may indicate a failed partial download, or it could be "
              "completely different files with the same name.")

        print("\nYou must provide input on how these files should be handled. "
              "If you answer Yes, the exsiting files will be DELETED and "
              "replaced with the contents on Box. If you answer No, "
              "the existing files will be left in place, and the copies on "
              "Box WILL NOT be downloaded.")

        choice = input("\nWould you like to overwrite these files? [y/N] ")
        if choice.upper() == 'N' or choice == '':
            # If the user DOES NOT want to overwrite exsiting partial
            # downloads, consider them "existing files" so they are
            # excluded from the download set.
            existing_files.extend(incorrect_files)

    return set(existing_files)

def main(args=None):
    global DESTINATION

    # TODO: parse arguments


    try:
        load_settings_file()
    except NoConfigFoundError:
        print("No configuration file found!")
        print(config_file_help)
        return 1

    print(f"""
Slrupbox 1.0 - A tool for downloading files from Box.

Configuration:
    Box user: {SETTINGS['remote']['username']}
    Destination directory: {SETTINGS['local']['destination']}

""")

    DESTINATION = Path(SETTINGS['local']['destination'])

    # TODO: adjust this to be prettier
    init_curl(SETTINGS['remote']['username'],
              SETTINGS['remote']['password'])

    folder = choose_folder()

    print(f"\nYou have chosen: {folder}")

    files = get_files(folder)

    existing_files = filter_existing_files(files)
    files_to_download = files.difference(existing_files)

    if len(files_to_download) > 0:
        print_file_report(folder, files_to_download)
    else:
        print("There are no new files to download!")
        return 0


    choice = input("\nWould you like to continue? [Y/n] ")

    if choice.upper() == 'Y' or choice == '':
        download_files(files_to_download)

    # TODO: print a report of what was done

    return 0

if __name__ == '__main__':
    main()



from lxml import etree


def process_prop(prop):
    def fix_name(name):
        return name.replace('{DAV:}', '')

    ret = {}
    for i in prop:
        ret[fix_name(i.tag)] = i.text

    return ret

def process_response(response):
    filename = None
    for i in response.iter("{DAV:}href"):
        filename = i.text

    for p in response.iter("{DAV:}prop"):
        ret = process_prop(p)

    ret['filename'] = filename
    return ret


def process_xml(xml_data):
    # tree = etree.parse('dirlist.xml')
    root = etree.fromstring(xml_data)

    # root = tree.getroot()

    ret = []
    for i in root.iter("{DAV:}response"):
        ret.append(process_response(i))

    return ret

if __name__ == '__main__':
    from pprint import pprint

    with open('dirlist.xml', 'rb') as f:
        pprint(process_xml(f.read()))



from hashlib import sha1 as SHA1


def sha1sum_from_file(filename):
    sha1 = SHA1()
    with open(filename, "rb") as inf:
        while True:
            buff = inf.read(1024 * 1024)
            if len(buff) == 0:
                break
            sha1.update(buff)

    return sha1.hexdigest()


if __name__ == '__main__':
    s = sha1sum_from_file('Summary.xls')
    print(s)



import pycurl
from .settings import SETTINGS
from tqdm import tqdm

LOGIN = None
BAR = None
BLAST = 0

def init_curl(username, password):
    global LOGIN
    LOGIN = ':'.join([
        username,
        password,
    ])

def header_func(header_line):
    header_line = header_line.decode('iso-8859-1') # from spec
    print(header_line)

def progress(download_t, download_d, upload_t, upload_d):
    global BAR
    global BLAST

    if download_t == 0:
        return
    else:
        if BAR is None:
            BAR = tqdm(total=download_t, unit='B', unit_scale=True)

    # BAR.update(n) expects n to be the number of bytes since
    # we last called it, however download_d is the total number of bytes
    # downloaded, so we need to adjust it..
    diff = download_d - BLAST
    BLAST = download_d

    BAR.update(diff)

def PROPFIND(url_part, out_obj):
    url = SETTINGS['remote']['root_path'] + url_part

    c = pycurl.Curl()
    c.setopt(c.URL, url)
    c.setopt(c.WRITEDATA, out_obj)
    # c.setopt(c.HEADERFUNCTION, header_func)
    # c.setopt(c.NOPROGRESS, False)
    # c.setopt(c.XFERINFOFUNCTION, progress)
    c.setopt(c.CUSTOMREQUEST, 'PROPFIND')
    c.setopt(c.USERPWD, LOGIN)
    #c.setopt(c.RANGE, '0-499')
    c.perform()
    c.close()

def download_file(url_part, out_obj):
    global BAR
    global BLAST

    url = SETTINGS['remote']['root_path'] + url_part

    c = pycurl.Curl()
    c.setopt(c.URL, url)
    c.setopt(c.WRITEDATA, out_obj)
    # c.setopt(c.HEADERFUNCTION, header_func)
    c.setopt(c.NOPROGRESS, False)
    c.setopt(c.XFERINFOFUNCTION, progress)
    # c.setopt(c.CUSTOMREQUEST, 'PROPFIND')
    c.setopt(c.USERPWD, LOGIN)
    #c.setopt(c.RANGE, '0-499')
    c.perform()
    c.close()

    BAR.close()
    BLAST = 0
    BAR = None



import yaml
from os.path import exists, expanduser
from pprint import pprint

class NoConfigFoundError(RuntimeError):
    pass

SETTINGS = {}

def find_config():
    paths = [
        './slurpbox.conf',
        '~/.slurpboxrc',
        '/etc/slurpbox/slurpbox.conf',
    ]

    for path in paths:
        p = expanduser(path)
        if exists(p):
            return p

    raise NoConfigFoundError()


def load_settings_file(filename=None):
    global SETTINGS

    if filename is None:
        filename = find_config()

    with open(filename) as inf:
        SETTINGS.update(yaml.load(inf))



from distutils.core import setup

setup(name='slurpbox',
      version='1.0',
      description='Download all files from a Box.com folder',
      author='Quasar Jarosz',
      author_email='quasar@uams.edu',
      url='http://pypi.python.org/pypi/Slurpbox/',
      license='LICENSE.txt',
      packages=['slurpbox'],
      scripts=['bin/slurpbox'],
      long_description=open('README.txt').read(),
      install_requires=[
          'pycurl >= 7.43.0.1',
          'PyYAML >= 3.12',
          'tqdm >= 4.19.5',
          'lxml >= 4.1.1',
      ]
     )



config_file_help = """
Slurpbox looks for its configuration first in the current directory (named
slurpbox.conf), then in the current user's home directory (named .slurpboxrc),
and finally in the default system location, /etc/slurpbox/slurpbox.conf.

The config file is a YAML file, which should take the following form:

# slurpbox.conf

remote:
    username: <username@host>
    password: <password>

    # The root WebDAV address; you probably don't want to change this
    root_path: https://dav.box.com/

local:
    # Where to store downloaded files
    destination: <location>
"""
